@@ -5,7 +5,7 @@
  | program/include/iniset.php                                            |
  |                                                                       |
  | This file is part of the RoundCube Webmail client                     |
- | Copyright (C) 2008, RoundCube Dev, - Switzerland                      |
+ | Copyright (C) 2008-2009, RoundCube Dev, - Switzerland                 |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -22,7 +22,7 @@
 
 
 // application constants
-define('RCMAIL_VERSION', '0.2-alpha');
+define('RCMAIL_VERSION', '0.2.2');
 define('RCMAIL_CHARSET', 'UTF-8');
 define('JS_OBJECT_NAME', 'rcmail');
 
@@ -30,6 +30,8 @@ if (!defined('INSTALL_PATH')) {
   define('INSTALL_PATH', dirname($_SERVER['SCRIPT_FILENAME']).'/');
 }
 
+define('RCMAIL_CONFIG_DIR', INSTALL_PATH . 'config');
+
 // make sure path_separator is defined
 if (!defined('PATH_SEPARATOR')) {
   define('PATH_SEPARATOR', (eregi('win', PHP_OS) ? ';' : ':'));
@@ -48,11 +50,15 @@ if (set_include_path($include_path) === false) {
   die('Fatal error: ini_set/set_include_path does not work.');
 }
 
+ini_set('error_reporting', E_ALL&~E_NOTICE);
+if  (isset($_SERVER['HTTPS'])) {
+   ini_set('session.cookie_secure', ($_SERVER['HTTPS'] && ($_SERVER['HTTPS'] != 'off'))?1:0);
+} else {
+   ini_set('session.cookie_secure', 0);
+}
 ini_set('session.name', 'roundcube_sessid');
 ini_set('session.use_cookies', 1);
-ini_set('session.gc_maxlifetime', 21600);
-ini_set('session.gc_divisor', 500);
-ini_set('error_reporting', E_ALL&~E_NOTICE);
+ini_set('session.only_use_cookies', 1);
 set_magic_quotes_runtime(0);
 
 // increase maximum execution time for php scripts
@@ -61,19 +67,35 @@ if (!ini_get('safe_mode')) {
   set_time_limit(120);
 }
 
+// set internal encoding for mbstring extension
+if(extension_loaded('mbstring'))
+  mb_internal_encoding(RCMAIL_CHARSET);
+	      
+
 /**
  * Use PHP5 autoload for dynamic class loading
  * 
  * @todo Make Zend, PEAR etc play with this
+ * @todo Make our classes conform to a more straight forward CS.
  */
 function __autoload($classname)
 {
   $filename = preg_replace(
-      array('/MDB2_(.+)/', '/Mail_(.+)/', '/^html_.+/', '/^utf8$/'),
-      array('MDB2/\\1', 'Mail/\\1', 'html', 'utf8.class'),
+      array('/MDB2_(.+)/',
+    	    '/Mail_(.+)/',
+	    '/^html_.+/',
+	    '/^utf8$/',
+	    '/html2text/'
+	),
+      array('MDB2/\\1',
+    	    'Mail/\\1',
+	    'html',
+	    'utf8.class',
+	    'lib/html2text'	// see #1485505
+	),
       $classname
   );
-  include_once $filename. '.php';
+  include $filename. '.php';
 }
 
 /**
@@ -95,4 +117,3 @@ require_once 'include/rcube_shared.inc';
 
 // set PEAR error handling (will also load the PEAR main class)
 PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, 'rcube_pear_error');
-
